import random as rnd
from typing import Callable


def random_arr(min: float, max: float, size: int) -> list[float]:
    return [(min + (max - min)*rnd.random()) for i in range(size)]


def identity(x):
    return x


def generate_samples(fn: Callable,
                     x_range: range,
                     x_noise: Callable = identity,
                     y_noise: Callable = identity) -> list[tuple[float, float]]:
    """Generates samples of a given function with optional noise"""
    # closure
    def gen_one(x):
        x_distorted = x_noise(x)
        y_distorted = y_noise(fn(x_distorted))
        y_distorted
        return (x_distorted, y_distorted)

    return [gen_one(x) for x in x_range]


def linear(a: float, b: float) -> Callable:
    """Generates a lambda of a linear function with given parameters"""
    return (lambda x: a*x + b)


def regress(samples, iterations=1000, lr=0.002):
    w = [1, 1]
    gradient = [0, 0]
    N = len(samples)

    for i in range(iterations):
        def fn_pred(x): return w[0]*x + w[1]

        gradient[0] = 2/N*sum([(fn_pred(x) - y)*x for (x, y) in samples])
        gradient[1] = 2/N*sum([(fn_pred(x) - y) for (x, y) in samples])

        w[0] -= lr * gradient[0]
        w[1] -= lr * gradient[1]

    return w
