from helper_functions import *
import data_operations as dop
import matplotlib.pyplot as plt
import random as rd
from sys import argv


def print_stat_data(stat_data):
    print(f"""
    Length: {stat_data.length}
    Avg: {stat_data.avg}
    Deviation: {stat_data.std_deviation}
    Min: {stat_data.min}
    Max: {stat_data.max}
    """)

def regress_rust():
    used_range = range(-20, 40)

    argv_present = len(argv) >= 3

    if not (argv_present):
        print("""The program will run with default values.
    Specialised usage:
    program <a: number> <b: number> <dontanimate: any>\n""")

    a = 1.3 if not argv_present else float(argv[1])
    b = -12 if not argv_present else float(argv[2])
    samples = generate_samples(linear(a, b),
                               used_range,
                               lambda x: x + (rd.random() - 0.5) * 1/4,
                               lambda y: y + (rd.random() - 0.5) * 30,
                               )

    # unzips a list of tuples into two lists
    x_sample, y_sample = zip(*samples)


    stats = dop.calculate_statistical_data(y_sample)
    print("Statistical data for y values:")
    print_stat_data(stats)


    animate = True
    if len(argv) >= 4:
        animate = False

    history = dop.linear_regression(samples, 2500, 0.002)



    # plot creation section #
    fig = plt.figure()

    pred_ax = plt.subplot(111)  # for updateable predicted function plot
    sample_ax = plt.subplot(111)  # for the sample scatterplot

    pred_line,  = pred_ax.plot([linear(1, 1)(i) for i in used_range])
    sample_ax.scatter(x_sample, y_sample)

    step = 10 if animate else len(history)

    # plot update loop #
    for (i, (a, b)) in enumerate(history[0:-1:step]):
        pred_ax.set_title(f"Iteration {i*step}: {a}x + {b}")
        pred_line.set_data([i for i in used_range], [linear(a, b)(i)
                                                     for i in used_range])
        fig.canvas.draw()
        plt.pause(0.01)
    plt.waitforbuttonpress()


if __name__ == "__main__":
    regress_rust()
